// $Id: us_phone.h,v 1.2 2017/04/09 20:56:40 leavens Exp leavens $
#ifndef US_PHONE_H
#define US_PHONE_H 1
#include <stdlib.h>
#include <stdbool.h>

#define AREA_LEN 3
#define EXCHANGE_LEN 3
#define SUB_LEN 4

typedef struct {
    int area[AREA_LEN];
    int exchange[EXCHANGE_LEN];
    int subscriber[SUB_LEN];
} us_phone_t;

// Make "us_phone" a synonym for a us_phone_t pointer type.
typedef us_phone_t *us_phone;

// requires: area, exchange, and sub are all null-terminated
//           strings that are allocated, and the length of area is AREA_LEN
//           and the length of exchange is EXCHANGE_LEN, and the length
//           of sub is SUB_LEN.
// modifies: stdout
// effect: creates a fresh us_phone_t object allocated on the heap,
//         initializes its fields from the corresponding arguments,
//         and returns a pointer to the new object
//         (or NULL if the new object cannot be allocated or
//         if some of the characters in area, exchange, or sub are not digits)
//         When malloc fails the message "malloc failed!" is printed on stdout.
//         When some charactr is not a digit, a message of the form
//         "'C' is not a decimal digit" is printed on stdout, where C is the
//         non-digit character.
extern us_phone make_us_phone(const char *area, const char *exchange,
			      const char *sub);

// requires: stdin and stdout are open; area_prompt and digits_prompt are
//           both allocated, null-terminated strings.
// modifies: stdin, stdout
// effect: prompts with area_prompt on stdout, then reads the area code.
//         Then prompts with digits_prompt on stdout and reads the phone number.
//         The phone number is expected to be in the format NNN-NNNN,
//         with a hyphen between the exchange (NNN) and the subscriber number
//         (NNNN), where each N is a decimal digit.
//         Then allocates a us_phone_t struct on the heap and initializes it
//         with the data read from stdin, and returns a pointer to that
//         newly allocated struct (or NULL if there was an error or
//         if some of the characters that should be digits are not).
//         When malloc fails the message "malloc failed!" is printed on stdout.
//         When some charactr is not a digit, a message of the form
//         "'C' is not a decimal digit" is printed on stdout, where C is the
//         non-digit character.
extern us_phone read_us_phone(const char *area_prompt,
			      const char *digits_prompt);

// requires: p is allocated (and not NULL)
// ensures: result is a null-terminated string representing the area code of p
extern char *area_code(us_phone p);

// requires: p is allocated (and not NULL)
// ensures: result is a string of the form "Exchange-Subscriber" where
//          Exchange is a string representing p's exchange and Subscriber
//          is a string representing p's subscriber number.
extern char *digits(us_phone p);

// requires: p is allocated (and not NULL)
// ensures: result is a string of the form:
//              "(Areacode)Exchange-Subscriber"
//          where Areacod is a string representing the area code of p,
//          Exchange is a string representing p's exchange and Subscriber
//          is a string representing p's subscriber number.
extern char *us_phone_to_string(us_phone p);

// requires: p1 and p2 are both allocated (and not NULL)
// ensures: result is true just when each field of p1 is equal to
//          the corresponding field of p2.
extern bool equal_us_phone(us_phone p1, us_phone p2);
#endif
